CCCL (CUDA C++ Core Libraries) provides: - CUB: device/block/warp-level GPU primitives (reduce, scan, sort, topk) - Thrust: high-level parallel algorithms (transform_reduce, sort, scan) - libcudacxx: CUDA C++ standard library (atomics, barriers, memory) - cudax: experimental features (memory resources, allocators) - Tuning policies: per-SM hardware-specific algorithm parameters Competition optimization vectors mapped to CCCL: - Output TPS (83% weight): warp_reduce, block_reduce, device_topk - Input TPS (14% weight): device_scan, block_load, prefetch - Cache TPS (3% weight): prefix caching strategy patterns - Memory (0.9 util): pooled/cached/buddy allocators Source: https://github.com/NVIDIA/cccl (shallow clone, HEAD only) License: Apache-2.0
43 lines
1.3 KiB
Plaintext
43 lines
1.3 KiB
Plaintext
//===----------------------------------------------------------------------===//
|
|
//
|
|
// Part of CUDASTF in CUDA C++ Core Libraries,
|
|
// under the Apache License v2.0 with LLVM Exceptions.
|
|
// See https://llvm.org/LICENSE.txt for license information.
|
|
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
|
// SPDX-FileCopyrightText: Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES.
|
|
//
|
|
//===----------------------------------------------------------------------===//
|
|
|
|
#include <cuda/experimental/__stf/stream/interfaces/slice_reduction_ops.cuh>
|
|
#include <cuda/experimental/__stf/stream/stream_ctx.cuh>
|
|
|
|
using namespace cuda::experimental::stf;
|
|
|
|
__global__ void add(int* ptr, int value)
|
|
{
|
|
*ptr = *ptr + value;
|
|
}
|
|
|
|
int main()
|
|
{
|
|
stream_ctx ctx;
|
|
|
|
int a = 17;
|
|
auto handle = ctx.logical_data(make_slice(&a, 1));
|
|
auto redux_op = std::make_shared<slice_reduction_op_sum<int>>();
|
|
ctx.task(handle.relaxed(redux_op))->*[](auto stream, auto s) {
|
|
add<<<1, 1, 0, stream>>>(s.data_handle(), 42);
|
|
};
|
|
|
|
ctx.task(handle.rw())->*[](auto stream, auto s) {
|
|
add<<<1, 1, 0, stream>>>(s.data_handle(), 1);
|
|
};
|
|
|
|
ctx.task(exec_place::host(), handle.read())->*[](auto stream, auto s) {
|
|
cuda_safe_call(cudaStreamSynchronize(stream));
|
|
EXPECT(s(0) == 17 + 42 + 1);
|
|
};
|
|
|
|
ctx.finalize();
|
|
}
|